/* GtkDatabox - An extension to the gtk+ library
 * Copyright (C) 1998 - 2005  Dr. Roland Bock
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public License
 * as published by the Free Software Foundation; either version 2.1
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */
#include <stdlib.h>
#include <unistd.h>
#include <stdio.h>

#include <gtk/gtk.h>
#include <gtkdatabox.h>
#include <math.h>
/* Some versions of math.h have a PI problem... */
#ifndef PI
#define PI 3.14159265358979323846
#endif

#define POINTS 200
gfloat *X;
gfloat *Z;
gfloat **Y;
GdkColor color;

/*----------------------------------------------------------------
 *  databox addremove
 *----------------------------------------------------------------*/
void add_data (GtkDatabox * box, gfloat * y, gint factor)
{
   gfloat freq = 1 + (10.0 * rand()/(RAND_MAX+1.0));
   gfloat offset = (0.2 * rand()/(RAND_MAX+1.0));
   gint i;
   gint index;

   for (i = 0; i < POINTS; i++)
   {
      y[i] = factor * 100.0 * sin (offset + freq * i / 100.);
   }
   color.red = 1 + (int) (65535.0 * rand()/(RAND_MAX+1.0));
   color.green = 1 + (int) (65535.0 * rand()/(RAND_MAX+1.0));
   color.blue = 1 + (int) (65535.0 * rand()/(RAND_MAX+1.0));
            
   index = gtk_databox_data_add (GTK_DATABOX (box), POINTS,
			     X, y, color, GTK_DATABOX_POINTS, 1);

   printf ("Added: %d\n", index);
}

static gboolean
addremove_idle_func (GtkDatabox * box)
{
   GtkDataboxValue min, max;
   gint index = (int) (10. * rand()/(RAND_MAX+1.0));
   
   gtk_databox_data_remove (box, index);
   printf ("Removed: %d\n", index);
   gtk_databox_rescale (box);
   gtk_databox_data_get_extrema (box, &min, &max);
   printf ("minX: %10f, minY:%10f, maxX:%10f, maxY:%10f\n", min.x, min.y, max.x, max.y);
   add_data (box, Y[index], index);
   
   sleep (1);
   gtk_databox_redraw (box);

   return (TRUE);
}

  
static void
create_addremove (void)
{
   GtkWidget *window = NULL;
   GtkWidget *box1;
   GtkWidget *box2;
   GtkWidget *close_button;
   GtkWidget *box;
   GtkWidget *label;
   GtkWidget *separator;
   gint i;

   window = gtk_window_new (GTK_WINDOW_TOPLEVEL);
   gtk_widget_set_size_request (window, 300, 300);

   g_signal_connect (GTK_OBJECT (window), "destroy",
		     G_CALLBACK (gtk_main_quit), NULL);

   gtk_window_set_title (GTK_WINDOW (window), "GtkDatabox: Basics");
   gtk_container_set_border_width (GTK_CONTAINER (window), 0);

   box1 = gtk_vbox_new (FALSE, 0);
   gtk_container_add (GTK_CONTAINER (window), box1);

   label =
      gtk_label_new
      ("About once per second one of the datasets\nwill be removed, recalculated and added again.\n(This is just for testing add and remove functions)");
   gtk_box_pack_start (GTK_BOX (box1), label, FALSE, FALSE, 0);
   separator = gtk_hseparator_new ();
   gtk_box_pack_start (GTK_BOX (box1), separator, FALSE, FALSE, 0);

   box = gtk_databox_new ();

   color.red = 16383;
   color.green = 16383;
   color.blue = 16383;
   gtk_databox_set_background_color (GTK_DATABOX (box), color);
   X = g_new0 (gfloat, POINTS);
   Y = g_new0 (gfloat *, 11);

   for (i = 0; i < POINTS; i++)
   {
      X[i] = i * 5;
   }
   for (i = 0; i < 10; i++)
   {
      Y[i] = g_new0 (gfloat, POINTS);
      add_data (GTK_DATABOX (box), Y[i], i);
   }
   Y[10] = g_new0 (gfloat, 4);
   Z     = g_new0 (gfloat, 4);
   
   Z[0] = -100.; Y[10][0] = -1100.;
   Z[1] = -100.; Y[10][1] = +1100.;
   Z[2] = +1100.; Y[10][2] = -1100.;
   Z[3] = +1100.; Y[10][3] = +1100.;
   

   color.red = 65535;
   color.green = 0;
   color.blue = 0;
   gtk_databox_data_add (GTK_DATABOX(box), 4, Z, Y[10], color, GTK_DATABOX_POINTS, 3);
   
   gtk_databox_rescale (GTK_DATABOX (box));

   gtk_box_pack_start (GTK_BOX (box1), box, TRUE, TRUE, 0);

   separator = gtk_hseparator_new ();
   gtk_box_pack_start (GTK_BOX (box1), separator, FALSE, TRUE, 0);

   box2 = gtk_vbox_new (FALSE, 10);
   gtk_container_set_border_width (GTK_CONTAINER (box2), 10);
   gtk_box_pack_end (GTK_BOX (box1), box2, FALSE, TRUE, 0);
   close_button = gtk_button_new_with_label ("close");
   g_signal_connect_swapped (GTK_OBJECT (close_button), "clicked",
			     G_CALLBACK (gtk_main_quit), GTK_OBJECT (box));
   gtk_box_pack_start (GTK_BOX (box2), close_button, TRUE, TRUE, 0);
   GTK_WIDGET_SET_FLAGS (close_button, GTK_CAN_DEFAULT);
   gtk_widget_grab_default (close_button);

   gtk_widget_show_all (window);

   gtk_idle_add ((GtkFunction) addremove_idle_func, box);
}

gint
main (gint argc, char *argv[])
{
   gtk_init (&argc, &argv);

   create_addremove ();
   gtk_main ();

   return 0;
}
