/***************************************************************************
 *            gtkdatabox_marker.c
 *
 *  Copyright  2005  J.C.Remis & Dr. Roland Bock
 ****************************************************************************/
/* GtkDatabox - An extension to the gtk+ library
 * Copyright (C) 1998 - 2005  Dr. Roland Bock
 */
/*
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU Library General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

#include <gtkdatabox.h>

typedef struct 
{
   GtkDataboxMarker *markers;
   gint length;
   GdkGC *data_gc;
   GdkGC *label_gc;
   GdkGC *dashed_gc;
}
GtkDataboxMarkerData;

void
gtk_databox_marker_draw (GtkDatabox * box, GtkDataboxData * data)
{
   GtkDataboxMarkerData *marker_data;
   GtkDataboxMarker *markers;
   gint length;
   GdkPoint points [3];
   gint16 x;
   gint16 y;
   GtkDataboxCoord coord;
   gint size;
   gint i;

   g_return_if_fail (data->draw_info != NULL);

   marker_data = (GtkDataboxMarkerData *) data->draw_info;
   markers = marker_data->markers;
   length = marker_data->length;

   size = (data->size < 3) ? 3 : data->size;

   /* We monitor whether the data->gc is new or has changed */
   if (marker_data->data_gc != data->gc) 
   {
      /* Then we create a modified gc for our purposes */
      marker_data->data_gc = data->gc;
      if (!marker_data->label_gc) 
         marker_data->label_gc = gdk_gc_new (box->pixmap);
      gdk_gc_copy(marker_data->label_gc, data->gc);
      gdk_gc_set_line_attributes (marker_data->label_gc, 1, 
                                  GDK_LINE_SOLID,GDK_CAP_ROUND,GDK_JOIN_ROUND);
      if (!marker_data->dashed_gc) 
         marker_data->dashed_gc = gdk_gc_new (box->pixmap);
      gdk_gc_copy(marker_data->dashed_gc, data->gc);
      gdk_gc_set_line_attributes (marker_data->dashed_gc, 1, 
                                  GDK_LINE_ON_OFF_DASH,GDK_CAP_ROUND,GDK_JOIN_ROUND);
   }
   
   for (i = 0; i < length; ++ i)
   {
      x = (gint16) ((data->X[i] - box->top_left.x) * box->factor.x);
      y = (gint16) ((data->Y[i] - box->top_left.y) * box->factor.y);

      switch (markers[i].type)
      {
         case GTK_DATABOX_MARKER_TRIANGLE:
            switch (markers[i].position) 
            {
               case GTK_DATABOX_MARKER_N:
                  coord.x = x;
                  coord.y = y - 2 - size / 2;
                  y = y - 2;
                  points[0].x = x;
                  points[0].y = y;
                  points[1].x = x - size / 2;
                  points[1].y = y - size;
                  points[2].x = x + size / 2;
                  points[2].y = y - size;
                  break;
               case GTK_DATABOX_MARKER_E:
                  coord.x = x + 2 + size / 2;
                  coord.y = y;
                  x = x + 2;
                  points[0].x = x;
                  points[0].y = y;
                  points[1].x = x + size;
                  points[1].y = y + size / 2;
                  points[2].x = x + size;
                  points[2].y = y - size / 2;
                  break;
               case GTK_DATABOX_MARKER_S:
                  coord.x = x;
                  coord.y = y + 2 + size / 2;
                  y = y + 2;
                  points[0].x = x;
                  points[0].y = y;
                  points[1].x = x - size / 2;
                  points[1].y = y + size;
                  points[2].x = x + size / 2;
                  points[2].y = y + size;
                  break;
               case GTK_DATABOX_MARKER_W:
                  coord.x = x - 2 - size / 2;
                  coord.y = y;
                  x = x - 2;
                  points[0].x = x;
                  points[0].y = y;
                  points[1].x = x - size;
                  points[1].y = y + size / 2;
                  points[2].x = x - size;
                  points[2].y = y - size / 2;
                  break;
            }
            gdk_draw_polygon (box->pixmap, marker_data->label_gc, 
                              TRUE, points, 3);
            break;
            /* End of GTK_DATABOX_MARKER_TRIANGLE */
         case GTK_DATABOX_MARKER_SOLID_LINE:
         case GTK_DATABOX_MARKER_DASHED_LINE:
            switch (markers[i].position) 
            {
               case GTK_DATABOX_MARKER_N:
                  coord.x = x;
                  coord.y = size;
                  points[0].x = x;
                  points[0].y = 0;
                  points[1].x = x;
                  points[1].y = box->size.y;
                  break;
               case GTK_DATABOX_MARKER_E:
                  coord.x = box->size.x - size;
                  coord.y = y;
                  points[0].x = 0;
                  points[0].y = y;
                  points[1].x = box->size.x;
                  points[1].y = y;
                  break;
               case GTK_DATABOX_MARKER_S:
                  coord.x = x;
                  coord.y = box->size.y - size;
                  points[0].x = x;
                  points[0].y = 0;
                  points[1].x = x;
                  points[1].y = box->size.y;
                  break;
               case GTK_DATABOX_MARKER_W:
                  coord.x = size;
                  coord.y = y;
                  points[0].x = 0;
                  points[0].y = y;
                  points[1].x = box->size.x;
                  points[1].y = y;
                  break;
            }
            if (markers[i].type == GTK_DATABOX_MARKER_SOLID_LINE)
            {
               gdk_draw_line (box->pixmap, marker_data->label_gc,
                              points[0].x, points[0].y,
                              points[1].x, points[1].y);
            } else
            {
               gdk_draw_line (box->pixmap, marker_data->dashed_gc,
                              points[0].x, points[0].y,
                              points[1].x, points[1].y);
            }
            break;
            /* End of GTK_DATABOX_MARKER_LINE */
      }

      if (markers[i].label)
      {
         gtk_databox_text_write_at (box, markers[i].label, 
                                    marker_data->label_gc, coord,
                                    markers[i].label_position,
                                    size / 2 + 3,
                                    markers[i].boxed);
      }
   }
}

gint 
gtk_databox_marker_set_markers (GtkDatabox * box, gint index, 
                                GtkDataboxMarker *markers, gint length)
{
   GtkDataboxData *data = gtk_databox_data_get (box, index);
   GtkDataboxMarkerData *marker_data;
      
   g_return_val_if_fail (data, -1);

   g_return_val_if_fail (markers, -1);
   g_return_val_if_fail (length, -1);

   marker_data = g_new0 (GtkDataboxMarkerData, 1);
   marker_data->length = length;
   marker_data->markers = markers;

   data->draw_info = (gpointer) marker_data;
   
   return 0;
}


gint gtk_databox_marker_get_markers (GtkDatabox * box, gint index, 
               GtkDataboxMarker **markers, gint *length)
{
   GtkDataboxData *data = gtk_databox_data_get (box, index);
   GtkDataboxMarkerData *marker_data;

   g_return_val_if_fail (data, -1);
   g_return_val_if_fail (data->draw_info, -1);

   marker_data = (GtkDataboxMarkerData *) data->draw_info;

   *markers = marker_data->markers;
   *length = marker_data->length;
   return 0;
}

void
gtk_databox_marker_cleanup (GtkDatabox * box, GtkDataboxData * data)
{
   GtkDataboxMarkerData *marker_data;
   
   g_return_if_fail (data);
   g_return_if_fail (data->draw_info);

   marker_data = (GtkDataboxMarkerData *)data->draw_info;
   if (marker_data->label_gc)
      g_object_unref (marker_data->label_gc);

   if (marker_data->dashed_gc)
      g_object_unref (marker_data->dashed_gc);

   g_free(data->draw_info);
   data->draw_info = NULL;
}
