/***************************************************************************
 *            gtkdatabox_text.c
 *
 *  Copyright (C) 2005  J.C.Remis & Dr. Roland Bock
 ****************************************************************************/
/* GtkDatabox - An extension to the gtk+ library
 * Copyright (C) 1998 - 2005  Dr. Roland Bock
 */
/*
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU Library General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */
 
#include <gtkdatabox.h>

typedef struct 
{
   GtkDataboxText *texts;
   gint length;
   GdkGC *data_gc;
   GdkGC *text_gc;
}
GtkDataboxTextData;

void gtk_databox_text_draw (GtkDatabox * box, GtkDataboxData * data)
{
   GtkDataboxTextData *text_data;
   GtkDataboxText *texts;
   GtkDataboxCoord coord;
   gint size;
   gint length;
   gint i;

   g_return_if_fail (data->draw_info != NULL);

   text_data = (GtkDataboxTextData *) data->draw_info;
   texts = text_data->texts;
   length = text_data->length;
   size = data->size;

   /* We monitor whether the data->gc is new or has changed */
   if (text_data->data_gc != data->gc) 
   {
      /* Then we create a modified gc for our purposes */
      text_data->data_gc = data->gc;
      if (!text_data->text_gc) 
         text_data->text_gc = gdk_gc_new (box->pixmap);
      gdk_gc_copy(text_data->text_gc, data->gc);
      gdk_gc_set_line_attributes (text_data->text_gc, 1, 
                                  GDK_LINE_SOLID,GDK_CAP_ROUND,GDK_JOIN_ROUND);
   }

   for (i = 0; i < length; ++ i)
   {
      coord.x = (gint16) ((data->X[i] - box->top_left.x) * box->factor.x);
      coord.y = (gint16) ((data->Y[i] - box->top_left.y) * box->factor.y);

      if (texts[i].text)
        gtk_databox_text_write_at (box, texts[i].text, text_data->text_gc,
                                   coord, texts[i].position, size, 
                                   texts[i].boxed); 
   }

}

gint
gtk_databox_text_write_at (GtkDatabox * box, PangoLayout *pl, GdkGC *gc, 
                           GtkDataboxCoord coord, 
                           GtkDataboxTextPosition position,
                           gint distance, 
                           gboolean boxed)
{
   gint hdist_text;
   gint vdist_text;
   gint hdist_box;
   gint vdist_box;

   gint width;
   gint height;

   gint offset = (boxed) ? 2 : 0;

   pango_layout_get_pixel_size (pl, &width, &height);

   switch (position)
   {
      case GTK_DATABOX_TEXT_N:  hdist_text = - width/2; 
                                vdist_text = - distance - offset - height;
                                hdist_box  = hdist_text - offset;
                                vdist_box  = vdist_text - offset;
                                break;
      case GTK_DATABOX_TEXT_NE: hdist_text = + distance + offset;
                                vdist_text = - distance - offset - height;
                                hdist_box  = hdist_text - offset;
                                vdist_box  = vdist_text - offset;
                                break;
      case GTK_DATABOX_TEXT_E:  hdist_text = + distance + offset;
                                vdist_text = - height/2;
                                hdist_box  = hdist_text - offset;
                                vdist_box  = vdist_text - offset;
                                break;
      case GTK_DATABOX_TEXT_SE: hdist_text = + distance + offset;
                                vdist_text = + distance + offset;
                                hdist_box  = hdist_text - offset;
                                vdist_box  = vdist_text - offset;
                                break;
      case GTK_DATABOX_TEXT_S:  hdist_text = - width/2;
                                vdist_text = + distance + offset;
                                hdist_box  = hdist_text - offset;
                                vdist_box  = vdist_text - offset;
                                break;
      case GTK_DATABOX_TEXT_SW: hdist_text = - distance - offset - width;
                                vdist_text = + distance + offset;
                                hdist_box  = hdist_text - offset;
                                vdist_box  = vdist_text - offset;
                                break;
      case GTK_DATABOX_TEXT_W:  hdist_text = - distance - offset - width;
                                vdist_text = - height/2;
                                hdist_box  = hdist_text - offset;
                                vdist_box  = vdist_text - offset;
                                break;
      case GTK_DATABOX_TEXT_NW: hdist_text = - distance - offset - width;
                                vdist_text = - distance - offset - height;
                                hdist_box  = hdist_text - offset;
                                vdist_box  = vdist_text - offset;
                                break;
      default:                  hdist_text = - width/2;
                                vdist_text = - height/2;
                                hdist_box  = hdist_text - offset;
                                vdist_box  = vdist_text - offset;
   }
   
   gdk_draw_layout (box->pixmap, gc, 
                    coord.x + hdist_text,
                    coord.y + vdist_text,
                    pl);

   if (boxed)
      gdk_draw_rectangle(box->pixmap, gc, FALSE, 
                         coord.x + hdist_box,
                         coord.y + vdist_box,
                         width + 3, 
                         height + 3);
                    
   return (0);
}

gint 
gtk_databox_text_set_texts (GtkDatabox * box, gint index, 
                            GtkDataboxText *texts, gint length)
{
   GtkDataboxData *data = gtk_databox_data_get (box, index);
   GtkDataboxTextData *text_data;

   g_return_val_if_fail (GTK_IS_DATABOX (box), -1);
   g_return_val_if_fail (data, -1);

   g_return_val_if_fail (texts, -1);
   g_return_val_if_fail (length, -1);

   text_data = g_new0 (GtkDataboxTextData, 1);
   text_data->length = length;
   text_data->texts = texts;

   data->draw_info = (gpointer) text_data;

   return 0;
}

gint 
gtk_databox_text_get (GtkDatabox * box, gint index, 
                      GtkDataboxText **texts, gint *length)
{
   GtkDataboxData *data = gtk_databox_data_get (box, index);
   GtkDataboxTextData *text_data;

   g_return_val_if_fail (data, -1);
   g_return_val_if_fail (data->draw_info, -1);

   text_data = (GtkDataboxTextData *) data->draw_info;

   *texts = text_data->texts;
   *length = text_data->length;

   return 0;
}

void
gtk_databox_text_cleanup (GtkDatabox * box, GtkDataboxData * data)
{
   GtkDataboxTextData *text_data;

   g_return_if_fail (data);
   g_return_if_fail (data->draw_info);
   
   text_data = (GtkDataboxTextData *) data->draw_info;
   if (text_data->text_gc)
      g_object_unref (text_data->text_gc);
   
   g_free(data->draw_info);

   data->draw_info = NULL;
}
